/*
	This is the third and final demonstration of input operations

	This is a rewrite of the demo from day 2 except that instead
	of passing the buck on exceptions, we choose to handle them.	
	
	
*/

import java.io.*;
import java.util.*;
public class InputDemo {

	public static BufferedReader keybd = new BufferedReader (
											new InputStreamReader(System.in));

	public static final int MAX_ELEMENTS = 1000;


	public static void main(String args[]){
	
		System.out.println("I/O Demo");
		
		int[] list = new int[MAX_ELEMENTS];
		int listSize = 0; // the logical size

		System.out.println("First demo: Reading a file of strings. Use \"test1.txt\"");
		BufferedReader br1 = getInputStream("What is the input file for test 1? ");
		inputFileTest1(br1);
		
		System.out.println("\nSecond Demo: Reading a one-per-line file of ints. Use \"test2.txt\"");
		BufferedReader br2 = getInputStream("What is the input file for test 2? ");
		inputFileTest2(br2);			
	
		System.out.println("\nThird Demo: Reading an arbitrary file of ints and storing into"+
							" an array.\n Use \"test3.txt\"");
		BufferedReader br3 = getInputStream("What is the input file for test 3? ");

		listSize = inputFileTest3(br3, list, listSize);

		System.out.println("\nThe array from test 3\n");
		for (int i = 0; i < listSize; i++)		
			System.out.println("list[" + i + "]= " + list[i]);
			
	}


	public static BufferedReader getInputStream(String prompt) {
	// This method asks the typist for a file name and returns a BufferedReader object
	// representing the input stream from that file
	// If the typist mistypes the file name, the exception will be handled
	// and the typist will be asked for another file
		FileInputStream fis = null;

		do {

			System.out.print(prompt);

			try { // attempt to do the following block of statements
				String fileName = keybd.readLine();
				File inFile = new File(fileName);
				fis = new FileInputStream(inFile);
				// if you get to this point, the attempt was successful
				// and you go to the line following the catch clauses.
				// Otherwise, when an exception occurs, you exit this
				// block and go to the appropriate exception handler				
				
			}
			catch (FileNotFoundException e) {
				System.out.println("Sorry, the file you named doesn't exist.");
				System.out.println("Please try again.");
			}
			// Note: Since the FileNotFoundException is a specific IOException
			// it must be listed first. Otherwise, a file not found exception
			// would be treated by the handler for the general io exception
			catch (IOException e) {
				System.out.println("General I/O Error: " + e);
	
			}
		} while (fis == null);
	
		return new BufferedReader (new InputStreamReader(fis));
	}


	public static void inputFileTest1(BufferedReader br) {
	// This is the simplest file reading example
	// It highlights the loop pattern associated with file reading
		
		// We replace the old version:							
		//     String nextLine = br.readLine();
		// by:
		String nextLine = null;
		try {
			nextLine = br.readLine();
		} catch (IOException e) {System.out.println("General I/O error: " + e); }
		
		
		int lineNum = 1;
			
		while (nextLine != null) {
		
			System.out.println("Line " + lineNum + ": " + nextLine);
			lineNum++;
								
			// replaced again here
			try {
				nextLine = br.readLine();
			} catch (IOException e) {System.out.println("General I/O error: " + e); }
		}		
	}
	
	
	public static void inputFileTest2Bad(BufferedReader br) {
	// This is an attempt at reading a file of int (and finding the sum)
	// Here, we use a simple try/catch over the entire method.
	// This addresses the compilation problem brought on by having
	// to deal with exceptions (that is, the method compiles) but
	// it isn't very useful in attempting to recover from the exceptions
	// The next method does a better job
		try {
			String nextLine = br.readLine();
			int sum = 0;
			
			while (nextLine != null) {
				int n = Integer.parseInt(nextLine);
				sum += n;
			
				nextLine = br.readLine();
			}
			System.out.println("The sum of the ints is " + sum);
		}
		catch (Exception e) {
			System.out.println("Something went wrong.");
			System.out.println("Better luck next time.");
		}
	}	

	public static void inputFileTest2(BufferedReader br) {
	// This is a better attempt at reading a file of int (and finding the sum)

		String nextLine = null;
		try {
			nextLine = br.readLine();
		} catch (IOException e) {System.out.println("General I/O error: " + e); }

		int sum = 0;
		int lineNumber = 1;
		
		while (nextLine != null) {

			// This try is not required for compilation but it
			// makes our program more robust by recognizing possible
			// problems with the file, handling those problems, and
			// letting us continue with the program
			try {		
				sum += Integer.parseInt(nextLine);
			}
			catch (NumberFormatException e) {
				System.out.println("Input error on line " + lineNumber + " of the file.");
				System.out.println("An int was expected but the line is \"" + nextLine + "\"");
				System.out.println("Skipping this line in the sum.");
			}
			
			lineNumber++;		
			
			// This try is required because IOException is checked
			try { 
				nextLine = br.readLine();
			} catch (IOException e) {System.out.println("General I/O error: " + e); }
		
		}
		System.out.println("The sum of the ints is " + sum);		
	}


	public static int inputFileTest3(BufferedReader br, 
									int[] arr, int arrSize) {
	// This method shows the correct way to do read a file of ints 
	// (or a file of doubles) via string tokenizing
	// As an added attraction (since we've already done tokenization),
	// this method shows you how to load an array!
									
		String nextLine = null;
		try {
			nextLine = br.readLine();
		} catch (IOException e) {System.out.println("General I/O error: " + e); }
		
		int sum = 0;
		int lineNumber = 1;
		
		while (nextLine != null) {
		
			StringTokenizer st = new StringTokenizer(nextLine);
			
			while (st.hasMoreTokens()) {
				String token = null;
				try {
					token = st.nextToken();
					int n = Integer.parseInt(token);
					sum += n;		
					arr[arrSize++] = n;
				}
				catch (NumberFormatException e) {// a problem parsing on the line
					System.out.println("Format error on line " + lineNumber + ": " + nextLine);
					System.out.println("The following token was not an int: " + token);
				}
			}
			
			try {
				nextLine = br.readLine();
			} catch (IOException e) {System.out.println("General I/O error: " + e); }			//lineNum++;
		}
		System.out.println("The sum of the ints is " + sum);
		
		return arrSize;		
	}


}
